/**
 * La clase Sensores representa un sistema experimental que mide cuatro 
 * senales electricas periodicas, de tipo cuadrado con ruido
 * 
 * @author (Michael) 
 * @version (Abril 2015)
 */
public class Sensores
{
    // almacenamiento de las senales de cada uno de los cuatro sensores
    // las unidades son voltios
    // el primer indice es el sensor y el otro representa las sucesivas medidas
    private double[][] senal;

    // instantes en que se hacen las medidas de las senales de los sensores
    private double[] tiempo;

    // indice de la medida actual
    int actual=0;

    /**
     * Constructor que crea el objeto con todas las medidas simuladas
     */
    public Sensores()
    {
        // creamos los arrays para 4 sensores y 61 medidas
        senal= new double[4][61];
        tiempo=new double[61];

        // amplitudes de los sensores, en voltios
        double[] amplitud={8, 9, 10, 6};
        // media de los sensores en voltios
        double[] media={0.0, 5.0, 2.0, -1.5};
        // desfase de cada senal
        int[] fase={0,5,3,7};
        // tiempo de la medida, en segundos
        double t=0.0001;
        // signo de la senal cuadrada
        int[] signo={1,1,1,1};
        // periodo de la senal cuadrada (en decenas de milisegundos)
        int[] periodo={20,24,28,22};
        // Bucle para cada medida; son 61 en total
        for (int i=0; i<=60; i++) {
            // anotamos el tiempo y lo aumentamos para la siguiente medida
            tiempo[i]=t;
            t+=0.0001;
            // bucle para cada sensor
            for (int s=0; s<4; s++) {
                // A cada semiperiodo cambiamos el signo
                if ((i+fase[s])%(periodo[s]/2)==0) {
                    signo[s]=-signo[s];
                }   
                // ruido simulado con una funcion seno
                double ruido=Math.sin(tiempo[i]*20000);
                // calculo final de la senal
                senal[s][i]=signo[s]*amplitud[s]+media[s]+ruido;
            }
        }
    }

    /**
     * Hace que la medida actual recogida mediante el metodo medida() 
     * sea la primera realizada
     */
    public void rebobina()
    {
        actual=0;
    }

    /**
     * Retorna en voltios el valor de la medida actual del sensor indicado
     * por el parametro numSensor, que debe estar entre 0 y 3
     */
    public double medida(int numSensor) {
        try {
	    return senal[numSensor][actual]; 
	} catch (IndexOutOfBoundsException e) {
	    return Double.NaN;
	}
    }

    /**
     * Retorna en segundos el tiempo actual al que corresponden las 
     * medidas obtenidas por el metodo medida()
     */
    public double tiempo() {
        return tiempo[actual];
    }

    /**
     * Si quedan mas medidas avanza la medida actual a la siguiente 
     * y retorna true; en caso contrario, retorna false
     */
    public boolean avanza() {
        if (actual<tiempo.length-1) {
            actual++;
            return true;
        } else {
            return false;
        }
    }
}
